/* HTML tags */

BODY {
 background-color: white;
 color: darkblue;
}

TD { color: darkblue; }

H1 { text-align: center; }

H3 { font-style: italic; }

HR {
 width: 85%;
 align: center;
}

.copyright { color: darkblue; }

/* Links */

:link { color: blue; }

:visited { color: purple; }

:active { color: red; }

.el:link { font-style: italic; }

/* Examples */

DIV.fragment {
 color: maroon;
}
